@@ -1,8 +1,8 @@
 // Copyright 2015 Ted Mielczarek. See the COPYRIGHT
 // file at the top-level directory of this distribution.
 
-use chrono::prelude::*;
-use memmap::Mmap;
+use debugid::{CodeId, DebugId};
+use memmap2::Mmap;
 use minidump::system_info::{Cpu, Os};
 use minidump::*;
 use minidump_common::format as md;
@@ -11,6 +11,7 @@ use num_traits::cast::FromPrimitive;
 use std::fs::File;
 use std::io::Read;
 use std::path::PathBuf;
+use std::time::SystemTime;
 
 fn get_test_minidump_path(filename: &str) -> PathBuf {
     let mut path = PathBuf::from(file!());
@@ -29,6 +30,11 @@ fn read_test_minidump<'a>() -> Result<Minidump<'a, Mmap>, Error> {
     Minidump::read_path(&path)
 }
 
+fn read_linux_minidump<'a>() -> Result<Minidump<'a, Mmap>, Error> {
+    let path = get_test_minidump_path("linux-mini.dmp");
+    Minidump::read_path(&path)
+}
+
 #[test]
 fn test_minidump_read_path() {
     read_test_minidump().unwrap();
@@ -57,22 +63,28 @@ fn test_module_list() {
     assert_eq!(modules[0].base_address(), 0x400000);
     assert_eq!(modules[0].size(), 0x2d000);
     assert_eq!(modules[0].code_file(), "c:\\test_app.exe");
-    assert_eq!(modules[0].code_identifier(), "45D35F6C2d000");
+    assert_eq!(
+        modules[0].code_identifier().unwrap(),
+        CodeId::new("45D35F6C2d000".to_string())
+    );
     assert_eq!(modules[0].debug_file().unwrap(), "c:\\test_app.pdb");
     assert_eq!(
         modules[0].debug_identifier().unwrap(),
-        "5A9832E5287241C1838ED98914E9B7FF1"
+        DebugId::from_breakpad("5A9832E5287241C1838ED98914E9B7FF1").unwrap()
     );
     assert!(modules[0].version().is_none());
 
     assert_eq!(modules[12].base_address(), 0x76bf0000);
     assert_eq!(modules[12].size(), 0xb000);
     assert_eq!(modules[12].code_file(), "C:\\WINDOWS\\system32\\psapi.dll");
-    assert_eq!(modules[12].code_identifier(), "411096CAb000");
+    assert_eq!(
+        modules[12].code_identifier().unwrap(),
+        CodeId::new("411096CAb000".to_string())
+    );
     assert_eq!(modules[12].debug_file().unwrap(), "psapi.pdb");
     assert_eq!(
         modules[12].debug_identifier().unwrap(),
-        "A5C3A1F9689F43D8AD228A09293889702"
+        DebugId::from_breakpad("A5C3A1F9689F43D8AD228A09293889702").unwrap()
     );
     assert_eq!(modules[12].version().unwrap(), "5.1.2600.2180");
 
@@ -138,8 +150,13 @@ fn test_misc_info() {
     assert_eq!(misc_info.raw.process_id(), Some(&3932));
     assert_eq!(misc_info.raw.process_create_time(), Some(&0x45d35f73));
     assert_eq!(
-        misc_info.process_create_time().unwrap(),
-        Utc.ymd(2007, 2, 14).and_hms(19, 13, 55)
+        misc_info
+            .process_create_time()
+            .unwrap()
+            .duration_since(SystemTime::UNIX_EPOCH)
+            .unwrap()
+            .as_secs(),
+        1171480435, // = 2007-02-14T19:13:55
     );
 }
 
@@ -294,3 +311,61 @@ fn test_empty_minidump() {
         Err(e) => assert_eq!(e, Error::MissingHeader),
     }
 }
+
+#[test]
+fn backwards_range() {
+    let data = include_bytes!("../../testdata/invalid-range.dmp");
+
+    match Minidump::read(&data[..]) {
+        Ok(f) => {
+            let _ = f.get_stream::<MinidumpLinuxMaps>().unwrap();
+        }
+        Err(e) => {
+            panic!("Expected to parse the header, got {:?}", e);
+        }
+    }
+}
+
+#[test]
+fn test_record_count_mac_info() {
+    let data = include_bytes!("../../testdata/invalid-record-count.dmp");
+
+    match Minidump::read(&data[..]) {
+        Ok(f) => {
+            let _ = f.get_stream::<MinidumpMacCrashInfo>();
+        }
+        Err(e) => {
+            panic!("Expected to parse the header, got {:?}", e);
+        }
+    }
+}
+
+#[test]
+fn test_linux_os_version() {
+    let dump = read_linux_minidump().unwrap();
+    let system_info = dump.get_stream::<MinidumpSystemInfo>().unwrap();
+
+    assert_eq!(system_info.os_parts().0, "4.9.60-linuxkit-aufs");
+    assert_eq!(
+        system_info.os_parts().1.unwrap(),
+        "#1 SMP Mon Nov 6 16:00:12 UTC 2017",
+    );
+}
+
+#[test]
+fn test_full_dump_memory() {
+    let path = get_test_minidump_path("full-dump.dmp");
+    let dump = Minidump::read_path(&path).unwrap();
+    let memory_list = dump.get_stream::<MinidumpMemory64List<'_>>().unwrap();
+    assert_eq!(memory_list.iter().count(), 54);
+    let blocks: Vec<_> = memory_list.iter().take(3).collect();
+    assert_eq!(blocks[0].base_address, 0x007FFE0000);
+    assert_eq!(blocks[0].size, 0x1000);
+    assert_eq!(blocks[0].bytes[0..8], [0, 0, 0, 0, 0, 0, 0xA0, 0x0F]);
+    assert_eq!(blocks[1].base_address, 0x007FFE9000);
+    assert_eq!(blocks[1].size, 0x1000);
+    assert_eq!(blocks[1].bytes[0..8], [0x48, 0x61, 0x6C, 0x54, 0, 0, 0, 0]);
+    assert_eq!(blocks[2].base_address, 0x9897D0D000);
+    assert_eq!(blocks[2].size, 0x3000);
+    assert_eq!(blocks[2].bytes[0..8], [0, 0, 0, 0, 0, 0, 0, 0]);
+}
